@@ -223,13 +223,10 @@ static shared_ptr<const SOARecordContent> loadRPZFromServer(Logr::log_t plogger,
         continue;
       }
 
-      // We want the full name in the SOA record
-      if (dr.d_type == QType::SOA) {
-        zone->setSOA(dr);
-      }
       dr.d_name.makeUsRelative(zoneName);
       if (dr.d_type == QType::SOA) {
         sr = getRR<SOARecordContent>(dr);
+        zone->setSOA(dr);
         continue;
       }
 
@@ -392,12 +389,10 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& primaries, const boost::opt
 {
   setThreadName("rec/rpzixfr");
   bool isPreloaded = sr != nullptr;
-  auto luaconfsLocal = g_luaconfs.getLocal();
-
   auto logger = g_slog->withName("rpz");
 
   /* we can _never_ modify this zone directly, we need to do a full copy then replace the existing zone */
-  std::shared_ptr<DNSFilterEngine::Zone> oldZone = luaconfsLocal->dfe.getZone(zoneIdx);
+  std::shared_ptr<DNSFilterEngine::Zone> oldZone = g_luaconfs.getLocal()->dfe.getZone(zoneIdx);
   if (!oldZone) {
     SLOG(g_log << Logger::Error << "Unable to retrieve RPZ zone with index " << zoneIdx << " from the configuration, exiting" << endl,
          logger->error(Logr::Error, "Unable to retrieve RPZ zone from configuration", "index", Logging::Loggable(zoneIdx)));
@@ -447,7 +442,8 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& primaries, const boost::opt
         incRPZFailedTransfers(polName);
       }
     }
-
+    // Release newZone before (long) sleep to reduce memory usage
+    newZone = nullptr;
     if (!sr) {
       sleep(refresh);
     }
@@ -456,6 +452,8 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& primaries, const boost::opt
   bool skipRefreshDelay = isPreloaded;
 
   for (;;) {
+    // Don't hold on to oldZone, it well be re-assigned after sleep in the try block
+    oldZone = nullptr;
     DNSRecord dr;
     dr.setContent(sr);
 
@@ -465,6 +463,7 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& primaries, const boost::opt
     else {
       sleep(refresh);
     }
+    auto luaconfsLocal = g_luaconfs.getLocal();
 
     if (luaconfsLocal->generation != configGeneration) {
       /* the configuration has been reloaded, meaning that a new thread
@@ -511,7 +510,7 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& primaries, const boost::opt
 
       if (luaconfsLocal->generation != configGeneration) {
         SLOG(g_log << Logger::Info << "A more recent configuration has been found, stopping the existing RPZ update thread for " << zoneName << endl,
-             logger->info(Logr::Info, "A more recent configuration has been found, stopping the existing RPZ update thread"))
+             logger->info(Logr::Info, "A more recent configuration has been found, stopping the existing RPZ update thread"));
         return;
       }
       oldZone = luaconfsLocal->dfe.getZone(zoneIdx);
