@@ -9,7 +9,7 @@
 #include "validate-recursor.hh"
 #include "secpoll.hh"
 
-#include <stdint.h>
+#include <cstdint>
 #ifndef PACKAGEVERSION
 #define PACKAGEVERSION getPDNSVersion()
 #endif
@@ -19,49 +19,54 @@ string g_security_message;
 
 void doSecPoll(time_t* last_secpoll, Logr::log_t log)
 {
-  if (::arg()["security-poll-suffix"].empty())
+  if (::arg()["security-poll-suffix"].empty()) {
     return;
+  }
 
   string pkgv(PACKAGEVERSION);
-  struct timeval now;
-  gettimeofday(&now, 0);
+  struct timeval now
+  {
+  };
+  Utility::gettimeofday(&now);
 
   /* update last_secpoll right now, even if it fails
      we don't want to retry right away and hammer the server */
   *last_secpoll = now.tv_sec;
 
-  SyncRes sr(now);
+  SyncRes resolver(now);
   if (g_dnssecmode != DNSSECMode::Off) {
-    sr.setDoDNSSEC(true);
-    sr.setDNSSECValidationRequested(true);
+    resolver.setDoDNSSEC(true);
+    resolver.setDNSSECValidationRequested(true);
   }
-  sr.setId("SecPoll");
+  resolver.setId("SecPoll");
 
   vector<DNSRecord> ret;
 
   string version = "recursor-" + pkgv;
   string qstring(version.substr(0, 63) + ".security-status." + ::arg()["security-poll-suffix"]);
 
-  if (*qstring.rbegin() != '.')
+  if (*qstring.rbegin() != '.') {
     qstring += '.';
+  }
 
   boost::replace_all(qstring, "+", "_");
   boost::replace_all(qstring, "~", "_");
 
   vState state = vState::Indeterminate;
   DNSName query(qstring);
-  int res = sr.beginResolve(query, QType(QType::TXT), 1, ret);
+  int res = resolver.beginResolve(query, QType(QType::TXT), 1, ret);
 
-  if (g_dnssecmode != DNSSECMode::Off && res) {
-    state = sr.getValidationState();
+  if (g_dnssecmode != DNSSECMode::Off && res != 0) {
+    state = resolver.getValidationState();
   }
 
   auto vlog = log->withValues("version", Logging::Loggable(pkgv), "query", Logging::Loggable(query));
   if (vStateIsBogus(state)) {
     SLOG(g_log << Logger::Error << "Failed to retrieve security status update for '" + pkgv + "' on '" << query << "', DNSSEC validation result was Bogus!" << endl,
          vlog->info(Logr::Error, "Failed to retrieve security status update", "validationResult", Logging::Loggable(vStateToString(state))));
-    if (g_security_status == 1) // If we were OK, go to unknown
+    if (g_security_status == 1) { // If we were OK, go to unknown
       g_security_status = 0;
+    }
     return;
   }
 
@@ -72,7 +77,7 @@ void doSecPoll(time_t* last_secpoll, Logr::log_t log)
   }
 
   string security_message;
-  int security_status = g_security_status;
+  int security_status = static_cast<int>(g_security_status);
 
   try {
     processSecPoll(res, ret, security_status, security_message);
