@@ -1,4 +1,7 @@
+#ifndef BOOST_TEST_DYN_LINK
 #define BOOST_TEST_DYN_LINK
+#endif
+
 #include <boost/test/unit_test.hpp>
 
 #include <cstdio>
@@ -25,17 +28,17 @@ struct Fixture
 {
   static std::shared_ptr<DNSRecordContent> makeLocalhostRootDRC()
   {
-    return DNSRecordContent::mastermake(QType::SOA, QClass::IN, "localhost. root 1 604800 86400 2419200 604800");
+    return DNSRecordContent::make(QType::SOA, QClass::IN, "localhost. root 1 604800 86400 2419200 604800");
   }
 
   static std::shared_ptr<DNSRecordContent> makeLocalhostDRC()
   {
-    return DNSRecordContent::mastermake(QType::NS, QClass::IN, "localhost.");
+    return DNSRecordContent::make(QType::NS, QClass::IN, "localhost.");
   }
 
   static std::shared_ptr<DNSRecordContent> makePtrDRC(const std::string& name)
   {
-    return DNSRecordContent::mastermake(QType::PTR, QClass::IN, name);
+    return DNSRecordContent::make(QType::PTR, QClass::IN, name);
   }
 
   static void addDomainMapFixtureEntry(SyncRes::domainmap_t& domainMap,
@@ -57,7 +60,7 @@ struct Fixture
   {
     domainMap[DNSName{name}] = SyncRes::AuthDomain{
       .d_records = {
-        DNSRecord(name, DNSRecordContent::mastermake(type, QClass::IN, address), type),
+        DNSRecord(name, DNSRecordContent::make(type, QClass::IN, address), type),
         DNSRecord(name, makeLocalhostDRC(), QType::NS),
         DNSRecord(name, makeLocalhostRootDRC(), QType::SOA),
       },
@@ -128,8 +131,8 @@ struct Fixture
       "localhost" + actualSearchSuffix,
       {DNSRecord("localhost" + actualSearchSuffix, makeLocalhostDRC(), QType::NS),
        DNSRecord("localhost" + actualSearchSuffix, makeLocalhostRootDRC(), QType::SOA),
-       DNSRecord("localhost" + actualSearchSuffix, DNSRecordContent::mastermake(QType::AAAA, QClass::IN, "::1"), QType::AAAA),
-       DNSRecord("localhost" + actualSearchSuffix, DNSRecordContent::mastermake(QType::A, QClass::IN, "127.0.0.1"), QType::A)});
+       DNSRecord("localhost" + actualSearchSuffix, DNSRecordContent::make(QType::AAAA, QClass::IN, "::1"), QType::AAAA),
+       DNSRecord("localhost" + actualSearchSuffix, DNSRecordContent::make(QType::A, QClass::IN, "127.0.0.1"), QType::A)});
     addDomainMapFixtureEntry(domainMap, "self" + actualSearchSuffix, QType::AAAA, "::1");
     addDomainMapFixtureEntry(
       domainMap,
